package com.xkrs.controller;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.dao.SysUserDao;
import com.xkrs.helper.FirePointQueryManager;
import com.xkrs.service.FirePointService;
import com.xkrs.service.StreetService;
import com.xkrs.service.impl.FirePointServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

@RestController
public class FirePointController {

    public static Logger log = LoggerFactory.getLogger(FirePointServiceImpl.class);

    private final Locale locale = LocaleContextHolder.getLocale();

    @Resource
    private FirePointService firePointService;

    @Resource
    private SysUserDao sysUserDao;

    @Resource
    private StreetService streetService;

    @Resource
    private FirePointQueryManager firePointQueryManager;

//    /**
//     * （Complete）
//     */
//    @RequestMapping(value = "/download/vipuserfirepoint", method = RequestMethod.GET)
//    public String downloadVipUserFirePoint(HttpServletResponse response, HttpServletRequest request) {
//        return firePointService.downloadVipUserFilePoint(request, response);
//    }

//    /**
//     * （Complete）
//     */
//    @RequestMapping(value = "/download/firepoint", method = RequestMethod.GET)
//    public String downloadFirePoint(HttpServletResponse response, HttpServletRequest request, @RequestParam(value = "ids") String ids) {
//        return firePointService.downloadFirePoint(request, response, ids);
//    }

    /**
     * 查询火点播报（无token）
     *
     * @param limit 最大数量
     */
    @GetMapping("/queryFirePointBroadcast")
    public String queryFirePointBroadcast(@RequestParam(value = "limit", required = false) Integer limit) {
        return firePointService.queryFirePointBroadcast(limit);
    }

    /**
     * 动态多条件查询火点列表(Complete)
     */
    @GetMapping("/queryFirePoint")
    public String queryFirePoint(@RequestHeader(value = "Authorization") String token, @RequestParam(value = "code", required = false) String code, @RequestParam(value = "startTime", required = false) String startTime, @RequestParam(value = "endTime", required = false) String endTime, @RequestParam(value = "satelliteType", required = false) String satelliteType, @RequestParam(value = "landType", required = false) String landType) {
        return firePointService.queryFirePoint(token, code, startTime, endTime, satelliteType, landType);
    }

    /**
     * 查询今天的火点信息(Complete)
     */
    @GetMapping("/selectTodayFirePoint")
    public String selectTodayFirePoint(@RequestHeader(value = "Authorization") String token, @RequestParam("countyCode") String countyCode) {
        return firePointService.selectTodayFirePoint(token, countyCode);
    }

    /**
     * 动态多条件查询火点信息(Complete)
     */
    @PostMapping("/selectFirePoint")
    public String selectFirePoint(@RequestHeader(value = "Authorization") String token, @RequestBody Map map) {
        String code = (String) map.get("cityCode");
        String startTime = (String) map.get("startTime");
        String endTime = (String) map.get("endTime");
        String satelliteType = (String) map.get("satelliteType");
        String landType = (String) map.get("landType");
        return firePointService.queryFirePoint(token, code, startTime, endTime, satelliteType, landType);
    }

    /**
     * 查询该省所有的市(Complete)
     */
    @GetMapping("/selectCityName")
    public String selectCityName(@RequestParam("code") String countyCode) {
        if (9 == countyCode.length()) {
            List<Map<String, String>> maps = new ArrayList<>();
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS, maps, locale);
        } else if ("0000".equals(countyCode.substring(2))) {
            List<Map<String, String>> maps = streetService.selectCityList(countyCode);
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS, maps, locale);
        } else if ("00".equals(countyCode.substring(4)) && !"0000".equals(countyCode.substring(2))) {
            List<Map<String, String>> maps = streetService.selectCountyList(countyCode);
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS, maps, locale);
        } else {
            List<Map<String, String>> maps = streetService.selectStreetList(countyCode);
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS, maps, locale);
        }
    }

    /**
     * 根据火点编码查询火点信息 (Complete)
     */
    @GetMapping("/selectFirePointByCode")
    public String selectFirePointByCode(@RequestParam("fireCode") String fireCode) {
        return firePointService.selectFirePointByCode(fireCode);
    }

}
